'use strict';
/**
 * an class to generate options for ECharts
 * @module option-generator
 * @see module:index
 */
const _ = require('lodash');
const numeral = require('numeral');
const util = require('../util/index');
const constant = require('../constant');

// const MIN_DATA_ZOOM_LIMIT = 20;

const DEFAULT_OPTION = {
  animation: false,
  theme: constant.DEFAULT_THEME_NAME,
  tooltip: {},
  toolbox: {
    feature: {},
  },
  legend: {
    left: 0,
  },
  textStyle: {
    fontFamily: '"Helvetica Neue", "Microsoft YaHei"',
  },
};

class OptionGenerator {
  constructor(extraOption) {
    const me = this;
    extraOption = extraOption || {};
    me._option = {};
    _.merge(me._option, DEFAULT_OPTION);
    _.merge(me._option, util.parseExtraOption(extraOption));
    this._configLayout();
    this.originalData = this._option.originalData || [];
  }

  configCoordinates(coordinates, horizontal, extraOption) {
    const me = this;
    const option = me._option;
    extraOption = extraOption || {};
    let dimensionAxis = 'xAxis';
    let measuresAxis = 'yAxis';
    if (horizontal) {
      const temp = measuresAxis;
      measuresAxis = dimensionAxis;
      dimensionAxis = temp;
    }
    option[dimensionAxis] = _.extend(option[dimensionAxis], {
      type: 'category',
      data: coordinates,
      axisLabel: {
        textStyle: {
          color: '#666',
        },
      },
    }, extraOption.category);

    if (extraOption.yAxis) {
      option[measuresAxis] = extraOption.yAxis;
    } else {
      option[measuresAxis] = [{
        type: 'value'
      }];
    }
    option[measuresAxis].forEach(item => {
      _.merge(item, {
        splitLine: {
          show: true,
          lineStyle: {
            color: '#f5f5f5',
          },
        },
        axisLine: {
          show: false
        },
        nameTextStyle: {
          color: '#999',
        },
        axisLabel: {
          textStyle: {
            color: '#666',
          },
        },
        nameLocation: 'middle', // start, middle, end
        nameGap: 44,
      });
    });
    _.merge(option.xAxis, this._configXLabel());
    _.merge(option[measuresAxis][0], {
      name: option['data-yAxisName'] || ''
    });
    if (_.isNumber(option['data-yAxisMax'])) {
      option[measuresAxis][0].max = option['data-yAxisMax'];
    }
    if (_.isNumber(option['data-yAxisMin'])) {
      option[measuresAxis][0].min = option['data-yAxisMin'];
    }
    if (_.isNumber(option['data-yAxisMaxRight']) && option[measuresAxis][1]) {
      option[measuresAxis][1].max = option['data-yAxisMaxRight'];
    }
    if (_.isNumber(option['data-yAxisMinRight']) && option[measuresAxis][1]) {
      option[measuresAxis][1].min = option['data-yAxisMinRight'];
    }

    if (option['data-showDataZoomInside']) {
      option.dataZoom = [{
        show: true,
        type: 'slider',
        start: _.isPlainObject(option['data-dataZoomInside']) ? option['data-dataZoomInside'].start : 0,
        end: _.isPlainObject(option['data-dataZoomInside']) ? option['data-dataZoomInside'].end : 100
      }];
    } else {
      option.xAxis.axisTick = { alignWithLabel: true };
    }
    return me;
  }

  configMeasures(type, measures, dataSet, extraSeriesOption) {
    const me = this;
    const option = me._option;
    option.series = [];
    _.each(measures, (measure) => {
      const colInfo = dataSet.colByName[measure];
      const measureSchema = _.find(dataSet.schema, { name: measure });
      const data = this.originalData.map(v => v[measure]);
      // TODO: y轴格式化
      if (option.yAxis) {
        option.yAxis = option.yAxis.map(o => (
          _.merge(o, {
            axisLabel: { formatter: this._yFormatter },
          })
        ));
      }
      option.series.push(_.extend({
        name: colInfo.comments,
        type,
        data,
        label: {
          normal: {
            show: option['data-showNormalLabel'] || false
          }
        },
        areaStyle: {
          normal: {
            opacity: option['data-showAreaStyle'] ? 1 : 0
          }
        },
        yAxisIndex: (extraSeriesOption.measuresRight && (extraSeriesOption.measuresRight.indexOf(measure) > -1)) ? 1 : 0,
        smooth: option['data-smooth'] || false,
      }, extraSeriesOption));
      if (data.length === 1) {
        // 只有一条数据时，显示一个点
        option.series.forEach(item => {
          item.symbol = 'circle';
        });
      }
      if (option['data-hideSymbol']) {
        // 关闭圆点
        option.series.forEach(item => {
          item.symbol = 'image://';
        });
      }
    });
    option.tooltip.formatter = (measures => params => {
      /*
        {
          color,
          data/value,
          dataIndex,
          name,
          seriesName,
        }
       */
      const content = params.map((param, i) => {
        const measure = measures[i];
        const formatData = dataSet.colValuesByName[measure];
        return `
          <div style="height:20px;">
            <span style="display:inline-block;border-radius:8px;width:8px;height:8px;background:${param.color};vertical-align:middle;"></span>
            <span style="display:inline-block;padding:0 5px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;vertical-align:middle;">${param.seriesName}:</span>
            <span style="display:inline-block;vertical-align:middle;">${formatData[param.dataIndex] || '-'}</span>
          </div>
        `;
      });

      return `
        <div style="padding:5px;border-radius:5px;font-size:12px;">
          <div style="font-size:14px;font-weight:700;">${params[0].name}</div>
          <div>
            ${content.join('')}
          </div>
        </div>
      `;
    })(measures);
    return me;
  }

  configLegend(colNames, dataSet) {
    const me = this;
    me._option.legend = me._option.legend ? me._option.legend : {};
    _.merge(me._option.legend, {
      data: _.map(colNames, colName => dataSet.colByName[colName].comments)
    });
    return me;
  }

  configLegendByData(data) {
    const me = this;
    me._option.legend = me._option.legend ? me._option.legend : {};
    _.merge(me._option.legend, {
      data,
    });
    return me;
  }

  toOption() {
    return this._option;
  }

  _yFormatter = v => {
    let label = v.toString();
    const f = '0,0';
    for (const o of constant.BEAUTIFY_Y) {
      if (v >= o.min) {
        label = `${numeral(v/o.min).format(f)}${o.label}`;
        return label;
      }
    }
    return label;
  }

  _configLayout() {
    switch (this._option.padding) {
      case 'large':
        this._option.grid = {
          left: 100,
          right: 100,
          bottom: 30,
        };
        break;
      case 'small':
        this._option.grid = {
          left: 50,
          right: 50,
          bottom: 30,
          containLabel: true,
        };
        break;
      case 'none':
        this._option.grid = {
          left: this._option['data-yAxisName'] ? 40 : 10,
          // 露出最后一个刻度
          right: 5,
          bottom: 30,
          containLabel: true,
        };
        break;
      default:
        break;
    }
  }

  _configXLabel() {
    const lineWidth = 75;
    return {
      axisLabel: {
        formatter: (v) => {
          const defaultWidth = util.guessTextWidth(v);
          if (defaultWidth > lineWidth) {
            const guessLen = Math.floor(lineWidth / 12);
            let start = guessLen;
            let first = v.toString().substr(0, start);
            while (util.guessTextWidth(`${first}...`) <= lineWidth) {
              first += v[start];
              start++;
            }
            return `${first}...`;
            // while (util.guessTextWidth(first) <= lineWidth) {
            //   first += v[start];
            //   start++;
            // }
            // let second = v.slice(start);
            // if (defaultWidth > lineWidth * 2) {
            //   second = v.substr(start, guessLen);
            //   while (util.guessTextWidth(`${second}...`) <= lineWidth) {
            //     second += v[start];
            //     start++;
            //   }
            //   second += '...';
            // }
            // return `${first}\n${second}`;
          }
          return v;
        },
      },
    };
  }
}

module.exports = OptionGenerator;
